import bpy
from math import radians
from bpy.props import *
from bpy.types import Operator

from .sk_utils import *
from ..utils import *

# 全てのシェイプキーごとにオブジェクトを生成
class LAZYSHAPEKEYS_OT_shape_keys_create_obj_from_all(Operator):
	bl_idname = "lazy_shapekeys.shape_keys_create_obj_from_all"
	bl_label = "Create Objects for All Shape Keys"
	bl_description = "Create a new object of that shape for each shape key.\nIt is convenient to display all shape keys in a list, or to make them into separate objects, edit them individually, and then combine them as shape keys again"
	bl_options = {'REGISTER', 'UNDO'}


	@classmethod
	def poll(cls, context):
		if bpy.context.active_object:
			obj = bpy.context.active_object
			if obj.type == "MESH":
				return obj.data.shape_keys

	def invoke(self, context,event):
		dpi_value = bpy.context.preferences.system.dpi

		return context.window_manager.invoke_props_dialog(self, width=dpi_value*3)


	def draw(self, context):
		layout = self.layout
		sc = bpy.context.scene
		crt_pr = sc.lazy_shapekeys.create_obj
		layout.prop(crt_pr,"use_translate")
		col = layout.column()
		col.use_property_split = True
		col.use_property_decorate = False
		col.active = crt_pr.use_translate

		col.prop(crt_pr,"translate_x")

		cols = col.column(align=True)
		cols.use_property_split = True
		cols.use_property_decorate = False
		cols.active = bool(not crt_pr.translate_z == 0)
		cols.prop(crt_pr,"translate_z")

		row = cols.row(align=True)
		row.active = bool(not crt_pr.up_column_by_folder)
		row.prop(crt_pr,"up_column_by_number")

		colz = cols.column(align=True)
		colz.use_property_split = False
		colz.prop(crt_pr,"up_column_by_folder")


		layout.separator()
		layout.prop(crt_pr,"new_obj_name")

		layout.separator()

		layout.prop(crt_pr,"use_text")
		col = layout.column()
		col.active = crt_pr.use_text
		col.use_property_split = True
		col.use_property_decorate = False
		col.prop(crt_pr,"text_size")
		col.prop(crt_pr,"text_loc_z")


	def execute(self, context):
		sc = bpy.context.scene
		crt_pr = sc.lazy_shapekeys.create_obj

		act_obj = bpy.context.object
		old_act = act_obj

		# 選択を無効化
		for obj in bpy.context.selected_objects:
			obj.select_set(False)

		new_obj_l = []

		z_value = 0
		x_trans_count = 1
		finished_l = []
		before_is_folder = False
		for i,sk in enumerate(act_obj.data.shape_keys.key_blocks):
			if i == 0:
				continue
			if is_folder(sk): # 前のアイテムがフォルダーだった場合、記憶する。
				before_is_folder = True
				create_text(sk, sk.name, None, z_value + crt_pr.translate_z, True)
				continue

			new_obj_l, z_value, x_trans_count, finished_l = create_new_shape_key_obj(self, i, old_act, new_obj_l, z_value, x_trans_count, finished_l, before_is_folder,sk)

			if not is_folder(sk):
				before_is_folder = False


		for obj in new_obj_l:
			obj.select_set(True)

		bpy.context.view_layer.objects.active = obj

		self.report({'INFO'}, "Create Objects [%s]" % len(finished_l))
		return{'FINISHED'}


# シェイプキーオブジェクトを作成
def create_new_shape_key_obj(self, i, old_act, new_obj_l, z_value, x_trans_count, finished_l, before_is_folder,sk):
	sc = bpy.context.scene
	crt_pr = sc.lazy_shapekeys.create_obj

	# Z位置を変更して行を変える
	if not crt_pr.translate_z == 0:
		if crt_pr.up_column_by_folder:
			if before_is_folder:
				z_value += crt_pr.translate_z
				x_trans_count = 0
		else:
			if (x_trans_count) % crt_pr.up_column_by_number == 0:
				z_value += crt_pr.translate_z
				x_trans_count = 0


	bpy.context.view_layer.objects.active = old_act
	old_act.select_set(True)

	# 複製
	bpy.ops.object.duplicate_move()


	# 新規オブジェクト設定
	if crt_pr.use_translate:
		bpy.ops.transform.translate(value=(crt_pr.translate_x*x_trans_count, 0, z_value),constraint_axis=(True, False, False),)
		# new_obj.location[0] = crt_pr.translate_x*x_trans_count
		# new_obj.location[2] = z_value
		x_trans_count += 1


	new_obj = bpy.context.selected_objects[0]
	new_obj_l += [new_obj]


	# シェイプキーの値の変更
	tgt_new_skeys = new_obj.data.shape_keys
	for i_temp,temp_sk in enumerate(tgt_new_skeys.key_blocks):
		temp_sk.value = 0

	tgt_sk = tgt_new_skeys.key_blocks[i]
	new_data_name = old_act.name +"_"+ tgt_sk.name
	if crt_pr.new_obj_name:
		nw_obj_name = crt_pr.new_obj_name.replace("{obj_name}",act_obj.name).replace("{sk_name}",tgt_sk.name)
	else:
		nw_obj_name = new_data_name
	new_obj.name = nw_obj_name
	tgt_sk.value = 1
	tgt_sk.mute = False
	set_parent_key_value(tgt_new_skeys.key_blocks,tgt_sk)
	mute_shapekey_driver(tgt_new_skeys)

	# その他のシェプキーを全て削除
	new_obj.shape_key_add(name='temp_combine')
	for shapeKey in new_obj.data.shape_keys.key_blocks:
		new_obj.shape_key_remove(shapeKey)
	finished_l += [new_obj]
	new_obj.select_set(False)


	# テキストを作成
	font_obj = create_text(tgt_sk, new_data_name, new_obj, z_value, False)


	print("Create [%s]" % nw_obj_name)

	return new_obj_l, z_value, x_trans_count, finished_l


def create_text(tgt_sk, new_data_name, new_obj, z_value, is_folder):
	sc = bpy.context.scene
	crt_pr = sc.lazy_shapekeys.create_obj

	if not crt_pr.use_text:
		return


	font_curve = bpy.data.curves.new(type="FONT", name=new_data_name)
	font_curve.body = tgt_sk.name
	font_curve.size = crt_pr.text_size
	font_curve.align_x = 'CENTER'

	font_obj = bpy.data.objects.new(name=new_data_name, object_data=font_curve)
	bpy.context.scene.collection.objects.link(font_obj)
	if is_folder:
		font_obj.location.x -= crt_pr.translate_x
		if crt_pr.up_column_by_folder:
			font_obj.location.z = z_value
	else:
		font_obj.parent = new_obj
	font_obj.location.z +=crt_pr.text_loc_z
	font_obj.rotation_euler[0] = radians(90)

	return font_obj
